import cnchar from 'cnchar'
import cncharOrder from 'cnchar-order'
import cncharTrad from 'cnchar-trad'
import {
  type StrokeTypeMaybeUnclean,
  type StrokeType
} from '../../src-common/stroke-encodings'
import { strokeTypeOverrides } from './stroke-type-overrides'
import { compat } from './blocks/index'
import { normalizeStroke } from './stroke-normalize'

cnchar.use(cncharOrder, cncharTrad)

/**
 * Query stroke shapes for the characters from cnchar and return them in a
 * shortened form.
 *
 * If the stroke count does not match the given expected count, return an empty
 * array instead.
 *
 * Some characters have an override defined, e.g. if cnchar data would be
 * inaccurate.
 *
 * Returns an empty list and prints an error if the expected length is not
 * matched.
 */
export function queryStrokeTypes (
  char: string,
  expectedLen?: number
): StrokeType[] {
  const compatChar = compat(char)
  const override =
    strokeTypeOverrides.get(char) ?? strokeTypeOverrides.get(compatChar)
  if (override !== undefined) {
    return override
  }
  return queryCncharStrokeTypes(compatChar)
}

function queryCncharStrokeTypes (char: string): StrokeType[] {
  const strokeShapes = cnchar.stroke(char, 'order', 'shape')[0]
  if (!Array.isArray(strokeShapes)) {
    return []
  }
  // if cnchar has multiple stroke types, only keep first, and map to a closed
  // set of strokes without duplicate stroke kinds
  return strokeShapes.map(t => normalizeStroke(
    t.split('|')[0] as StrokeTypeMaybeUnclean
  ))
}
