import { StrokeType } from '../../src-common/stroke-encodings'
import { queryStrokeTypes } from '../../build/gen-hanzi-data/stroke-type'
import { cjkRadicals, kangxiRadicals } from './radicals'

describe('Kangxi radicals', () => {
  describe('have non-empty strokes', () => {
    for (const radical of kangxiRadicals) {
      test(radical, () => {
        const strokes = queryStrokeTypes(radical)
        expect(strokes.length).toBeGreaterThan(0)
      })
    }
  })
  const radicalsAgainstStrokes = [
    { char: '⼆', strokes: [StrokeType.HENG, StrokeType.HENG] }
  ]
  for (const { char, strokes } of radicalsAgainstStrokes) {
    expect(queryStrokeTypes(char)).toEqual(strokes)
  }
})

describe('CJK Unified Ideograph radicals', () => {
  describe('have non-empty strokes', () => {
    for (const radical of cjkRadicals) {
      test(radical, () => {
        const strokes = queryStrokeTypes(radical)
        expect(strokes.length).toBeGreaterThan(0)
      })
    }
  })
  const radicalsAgainstStrokes = [
    { char: '二', strokes: [StrokeType.HENG, StrokeType.HENG] }
  ]
  for (const { char, strokes } of radicalsAgainstStrokes) {
    expect(queryStrokeTypes(char)).toEqual(strokes)
  }
})

describe('CJK Radicals Supplement radicals', () => {
  // note: for the moment we accept that some do not yet have strokes
  /* describe('have non-empty strokes', () => {
    for (const radical of cjkRadicalsSupplement) {
      test(radical, () => {
        const strokes = queryStrokeTypes(radical)
        expect(strokes.length).toBeGreaterThan(0)
      })
    }
  }) */
  const radicalsAgainstStrokes = [
    { char: '⺅', strokes: [StrokeType.PIE, StrokeType.SHU] }
  ]
  for (const { char, strokes } of radicalsAgainstStrokes) {
    expect(queryStrokeTypes(char)).toEqual(strokes)
  }
})
